<?php
/**
 *
 * CashBill Payment PHP SDK
 *
 * @author Lukasz Firek <lukasz.firek@cashbill.pl>
 * @version 1.0.0
 * @license MIT
 * @copyright CashBill S.A. 2015
 * 
 * http://cashbill.pl
 * 
 */
namespace CashBill\Payments\Model\Library;

use CashBill\Payments\Model\Library\Services\CurlConnection;
use CashBill\Payments\Model\Library\Services\CashBillException;
use CashBill\Payments\Model\Library\Services\CashBillConfigurationException;
use CashBill\Payments\Model\Library\Services\CashBillCurlException;
use CashBill\Payments\Model\Library\Services\CashBillPersonalDataException;
use CashBill\Payments\Model\Library\Services\CashBillTransactionException;
use CashBill\Payments\Model\Library\Model\PaymentChannels;
use CashBill\Payments\Model\Library\Model\Transaction;
use CashBill\Payments\Model\Library\Model\Amount;
use CashBill\Payments\Model\Library\Model\PersonalData;
use CashBill\Payments\Model\Library\Helpers\SignatureGenerator;

/**
 * Class Shop
 *
 * @package CashBill\Payments\Model\Library
 */
class Shop {
	const productionEndpoint = 'https://pay.cashbill.pl/ws/rest/';
	const testEndpoint = 'https://pay.cashbill.pl/testws/rest/';
	/**
	 *
	 * @var string $pointId
	 * @var string $pointKey
	 * @var CurlConnection $curlConnection
	 * @var CashBillPaymentChannels $paymentChannels
	 * @var CashBillTransaction $transaction
	 * @var CashBillPersonalData $personalData
	 */
	private $pointId, $pointKey, $curlConnection, $paymentChannels;
	
	/**
	 *
	 * @param string $pointId        	
	 * @param string $pointKey        	
	 * @param boolean $production        	
	 * @throws CashBillConfigurationException
	 */
	function __construct($pointId, $pointKey, $production = false) {
		if (empty ( $pointId ) || empty ( $pointKey )) {
			throw new CashBillConfigurationException( "Configuration required params not set" );
		}
		
		if (strlen ( $pointKey ) != 32) {
			throw new CashBillConfigurationException ( "Payment Point Key invalid value" );
		}
		
		$this->pointId = $pointId;
		$this->pointKey = $pointKey;
		$this->curlConnection = $production == true ? new CurlConnection ( self::productionEndpoint ) : new CurlConnection ( self::testEndpoint );
	}
	
	/**
	 *
	 * @param string $lang        	
	 * @throws CashBillException
	 * @return array
	 */
	public function getPaymentChannels($lang = 'pl') {
		if ($this->paymentChannels instanceof CashBillPaymentChannels) {
			return $this->paymentChannels->getAllPaymentChannels ();
		}
		try {
			
			$paymentChannels = $this->curlConnection->get ( "paymentchannels/{$this->pointId}/{$lang}" );
		} catch ( CashBillCurlException $exception ) {
			throw new CashBillException ( "Payment Channels Get Exception " . $exception->getMessage () );
		}
		
		$this->paymentChannels = new PaymentChannels ( $paymentChannels, $lang );
		
		return $this->paymentChannels->getAllPaymentChannels ();
	}
	/**
	 * 
	 * @param string $title
	 * @param CashBillAmount | array $amount
	 * @param string $description
	 * @param string $additionalData
	 * @param string $personalData
	 * @param string $returnUrl
	 * @param string $negativeReturnUrl
	 * @param string $paymentChannel
	 * @param string $languageCode
	 * @param string $referer
	 * @throws CashBillException
	 * @return \CashBill\Payments\Model\Library\Services\mixed
	 */
	public function createPayment($title, $amount, $description, $additionalData, $personalData = null, $returnUrl = null, $negativeReturnUrl = null, $paymentChannel = null, $languageCode = null, $referer = null) {
		
		try {
			
		if (! ($amount instanceof Amount)) {
			$amount = Amount::fromArray($amount);
		}
		
		$transactionData = new Transaction ( $title, $amount, $description, $additionalData );
		if ($paymentChannel != null) {
			$transactionData->setPaymentChannel ( $paymentChannel );
		}
		
		if ($referer != null) {
			$transactionData->setReferer ( $referer );
		}
		
		if ($returnUrl != null) {
			if ($negativeReturnUrl != null) {
				$transactionData->setReturnUrls ( $returnUrl, $negativeReturnUrl );
			} else {
				$transactionData->setReturnUrls ( $returnUrl );
			}
		}
		
		if ($languageCode != null) {
			$transactionData->setLanguage ( $languageCode );
		}
		
		if ($personalData == null) {
			$transactionData = $transactionData->toArray ();
			$transactionData ['sign'] = SignatureGenerator::generateSHA1 ( $transactionData, $this->pointKey );
		} else {
			
			if (! ($personalData instanceof PersonalData)) {
			$personalData = PersonalData::fromArray($personalData);
			}
			
			$transactionData = array_merge ( $transactionData->toArray (), $personalData->toArray () );
			$transactionData ['sign'] = SignatureGenerator::generateSHA1 ( $transactionData, $this->pointKey );
		}
		
		return $this->curlConnection->post ( "payment/{$this->pointId}", $transactionData );
		
		} catch ( CashBillException $exception ) {
			throw new CashBillException ( "Create Payment Get Exception " . $exception->getMessage () );
		}
	}
	
	/**
	 *
	 ** @param string $orderId   
	 * @param string $returnUrl        	
	 * @param string $negativeReturnUrl        	
	 * @throws CashBillException
	 * @return boolean
	 */
	public function changeRedirectUrl($orderId, $returnUrl, $negativeReturnUrl = '') {
		
		if(empty($orderId))
		{
			throw CashBillException ( "orderId is empty" );
		}
		
		if (empty ( $returnUrl )) {
			throw CashBillException ( "returnUrl is empty" );
		}
		
		if (! filter_var ( $returnUrl, FILTER_VALIDATE_URL )) {
			throw CashBillException ( "returnUrl not valid" );
		}
		
		if (! empty ( $negativeReturnUrl ) && ! filter_var ( $returnUrl, FILTER_VALIDATE_URL )) {
			throw CashBillException ( "negativeReturnUrl not valid" );
		}
		
		$changeData = array (
				'id' => $orderId,
				'returnUrl' => $returnUrl,
				'negativeReturnUrl' => $negativeReturnUrl 
		);
		
		$changeData ['sign'] = SignatureGenerator::generateSHA1 ( $changeData, $this->pointKey );
		
		try {
			$this->curlConnection->put ( "payment/{$this->pointId}/{$changeData['id']}", $changeData );
			return true;
		} catch ( CashBillCurlException $exception ) {
			throw new CashBillException ( "Url Change Exception " . $exception->getMessage () );
		}
	}
	
	/**
	 *
	 * @param string $orderId        	
	 * @throws CashBillException
	 * @return stdClass
	 */
	public function getFullPaymentInfo($orderId) {
		$paymentData = array (
				'sign' => SignatureGenerator::generateSHA1 ( $orderId, $this->pointKey ) 
		);
		
		try {
			
			return $this->curlConnection->get ( "payment/{$this->pointId}/{$orderId}", $paymentData );
		} catch ( CashBillCurlException $exception ) {
			throw new CashBillException ( "Payment Info Exception " . $exception->getMessage () );
		}
	}
	
	/**
	 *
	 * @param string $orderId        	
	 * @throws CashBillException
	 * @return boolean
	 */
	public function isPaid($orderId) {
		$paymentData = array (
				'sign' => SignatureGenerator::generateSHA1 ( $orderId, $this->pointKey ) 
		);
		
		try {
			
			$transaction = $this->curlConnection->get ( "payment/{$this->pointId}/{$orderId}", $paymentData );
			
			if ($transaction->status == "PositiveFinish") {
				return true;
			} else {
				
				return false;
			}
		} catch ( CashBillCurlException $exception ) {
			throw new CashBillException ( "Payment Info Exception : " . $exception->getMessage () );
		}
	}
	
	/**
	 *
	 * @param string $finishStatusNotify
	 * @param string $anotherStatusNotify       	
	 * @param string $class        	
	 */
	public function notificationHandler($finishStatusNotify, $anotherStatusNotify = null, $class = null) {
		if (isset ( $_GET ['cmd'] ) && isset ( $_GET ['args'] ) && isset ( $_GET ['sign'] )) {
			
			if (SignatureGenerator::generateMD5 ( $_GET ['cmd'] . $_GET ['args'], $this->pointKey ) == $_GET ['sign']) {
				
				if ($this->isPaid ( $_GET ['args'] )) {
					if ($class == null) {
						call_user_func ( $finishStatusNotify, $this->getFullPaymentInfo ( $_GET ['args'] ) );
					} else {
						call_user_func ( array (
								$class,
								$finishStatusNotify 
						), $this->getFullPaymentInfo ( $_GET ['args'] ) );
					}
				} else {
					if ($anotherStatusNotify != null) {
						if ($class == null) {
							call_user_func ( $anotherStatusNotify, $this->getFullPaymentInfo ( $_GET ['args'] ) );
						} else {
							call_user_func ( array (
									$class,
									$anotherStatusNotify 
							), $this->getFullPaymentInfo ( $_GET ['args'] ) );
						}
					}
				}
				echo "OK";
				exit ();
			} else {
				echo "SIGNATURE ERROR";
				exit ();
			}
		}
	}
}
